<?php

namespace Drupal\firebase_ui\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Component\Datetime\Time;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns a table list of Firebase notifications.
 */
class NotificationListController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\Time
   */
  protected $time;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs the NotificationListController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Component\Datetime\Time $time
   *   The time service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    DateFormatterInterface $date_formatter,
    Time $time,
    Connection $database,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->dateFormatter = $date_formatter;
    $this->time = $time;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('date.formatter'),
      $container->get('datetime.time'),
      $container->get('database'),
    );
  }

  /**
   * Renders a list of Firebase notifications in table format.
   *
   * @return array
   *   A render array.
   */
  public function list(): array {
    if (!$this->database->schema()->tableExists('firebase_notification')) {
      return [
        '#markup' => $this->t('The Firebase Notification table does not exist yet. Please run <code>drush entity:updates</code>.'),
      ];
    }

    $header = [
      'id' => $this->t('#'),
      'type' => $this->t('Type'),
      'title' => $this->t('Title'),
      'message' => $this->t('Message'),
      'send_time' => $this->t('Send Time'),
      'created' => $this->t('Created At'),
      'status' => $this->t('Status'),
      'actions' => $this->t('Actions'),
    ];

    $rows = [];
    $storage = $this->entityTypeManager->getStorage('firebase_notification');
    $notifications = $storage->loadMultiple();
    $now = $this->time->getCurrentTime();

    foreach ($notifications as $notification) {
      $send_time_value = (int) $notification->get('send_time')->value;
      $created_value = (int) $notification->get('created')->value;

      $send_time = ($send_time_value <= $created_value)
        ? $this->t('Immediately')
        : $this->dateFormatter->format($send_time_value, 'custom', 'Y-m-d H:i');

      $links = [];
      $is_future = $send_time_value > $now;

      if ($is_future) {
        $toggle_status = $notification->get('status')->value ? 'pause' : 'resume';
        $links[] = Link::fromTextAndUrl(ucfirst($toggle_status), Url::fromRoute('firebase_ui.notification_toggle', [
          'firebase_notification' => $notification->id(),
        ]))->toRenderable();
      }

      if (!$is_future) {
        $links[] = Link::fromTextAndUrl('Resend', Url::fromRoute('firebase_ui.notification_resend', [
          'firebase_notification' => $notification->id(),
        ]))->toRenderable();
      }

      $actions = [];
      foreach ($links as $index => $link) {
        $actions[] = $link;
        if ($index < count($links) - 1) {
          $actions[] = ['#markup' => ' | '];
        }
      }

      $status = ($send_time_value <= $now)
        ? $this->t('Completed')
        : ($notification->get('status')->value ? $this->t('Active') : $this->t('Paused'));

      $rows[] = [
        'id' => $notification->id(),
        'type' => ucfirst($notification->get('type')->value),
        'title' => $notification->get('title')->value,
        'message' => $notification->get('message')->value,
        'send_time' => $send_time,
        'created' => $this->dateFormatter->format($created_value, 'custom', 'Y-m-d H:i'),
        'status' => $status,
        'actions' => [
          'data' => [
            '#type' => 'container',
            'links' => $actions,
          ],
        ],
      ];
    }

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No notifications found.'),
    ];
  }

}
