<?php

namespace Drupal\firebase_ui\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Controller\ControllerBase;

/**
 * Returns the dynamic service worker JavaScript for authenticated users.
 */
class ServiceWorkerController extends ControllerBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new ServiceWorkerController.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(AccountInterface $current_user, ConfigFactoryInterface $config_factory) {
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('config.factory')
    );
  }

  /**
   * Outputs the dynamic Firebase Service Worker JavaScript.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   A JavaScript response with Firebase configuration.
   */
  public function sw() {
    // Only allow logged-in users.
    if ($this->currentUser->isAnonymous()) {
      return new Response('// Service Worker: Access denied', 403, [
        'Content-Type' => 'application/javascript',
      ]);
    }

    // Load Firebase config from Drupal settings.
    $config = $this->configFactory->get('firebase_ui.settings');

    $firebase_config = json_encode([
      'appId' => $config->get('appId'),
      'apiKey' => $config->get('apiKey'),
      'projectId' => $config->get('projectId'),
      'messagingSenderId' => $config->get('messagingSenderId'),
      'authDomain' => $config->get('authDomain') . ".firebaseapp.com",
    ], JSON_UNESCAPED_SLASHES);

    $js = <<<JS
importScripts('https://www.gstatic.com/firebasejs/10.7.0/firebase-app-compat.js');
importScripts('https://www.gstatic.com/firebasejs/10.7.0/firebase-messaging-compat.js');

firebase.initializeApp($firebase_config);

const messaging = firebase.messaging();
messaging.onBackgroundMessage(function(payload) {
  self.registration.showNotification(payload.notification.title, {
    body: payload.notification.body,
    icon: '/logo.png'
  });
});
JS;

    return new Response($js, 200, ['Content-Type' => 'application/javascript']);
  }

}
