<?php

namespace Drupal\firebase_ui\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Datetime\TimeInterface;
use Drupal\firebase_ui\Entity\FirebaseNotification;
use Drupal\firebase_ui\Service\NotificationSender;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes items in the firebase_ui_notification_queue.
 *
 * @QueueWorker(
 *   id = "firebase_ui_notification_queue",
 *   title = @Translation("Firebase UI Notification Queue"),
 *   cron = {"time" = 30}
 * )
 */
class FirebaseNotificationQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The time service.
   *
   * @var \Drupal\Core\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The queue factory service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The notification sender service.
   *
   * @var \Drupal\firebase_ui\Service\NotificationSender
   */
  protected $notificationSender;

  /**
   * Constructs a new FirebaseNotificationQueueWorker.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory service.
   * @param \Drupal\firebase_ui\Service\NotificationSender $notification_sender
   *   The notification sender service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, TimeInterface $time, QueueFactory $queue_factory, NotificationSender $notification_sender) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->time = $time;
    $this->queueFactory = $queue_factory;
    $this->notificationSender = $notification_sender;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('time'),
      $container->get('queue'),
      $container->get('firebase_ui.notification_sender')
    );
  }

  /**
   * Processes a single queue item.
   *
   * @param mixed $data
   *   The queue data array containing 'nid'.
   */
  public function processItem($data) {
    if (empty($data['nid'])) {
      return;
    }

    $storage = $this->entityTypeManager->getStorage('firebase_notification');
    $notification = $storage->load($data['nid']);

    if (!$notification || !$notification->get('status')->value) {
      return;
    }

    $current_time = $this->time->getCurrentTime();
    $send_time = $notification->get('send_time')->value;

    if ($send_time > $current_time) {
      // Requeue for later.
      $this->queueFactory->get('firebase_ui_notification_queue')->createItem([
        'nid' => $notification->id(),
      ]);
      return;
    }

    // Send to users.
    $users = $this->getTargetUsers($notification);

    foreach ($users as $user) {
      // Loop through tokens for the user.
      $tokens = $user->get('field_firebase_tokens')->getValue();
      foreach ($tokens as $tokenItem) {
        $token = $tokenItem['value'];
        $this->notificationSender->sendNotification(
          $token,
          $notification->get('title')->value,
          $notification->get('message')->value,
          [
            'type' => 'alert',
            'user_id' => $user->id(),
          ]
        );
      }
    }
  }

  /**
   * Gets all applicable users for the notification.
   *
   * @param \Drupal\firebase_ui\Entity\FirebaseNotification $notification
   *   The Firebase notification entity.
   *
   * @return \Drupal\user\UserInterface[]
   *   An array of loaded user entities.
   */
  private function getTargetUsers(FirebaseNotification $notification) {
    $type = $notification->get('type')->value;
    $target_ids = array_filter(explode(',', $notification->get('target_ids')->value));
    $uids = [];

    if ($type === 'all') {
      $all_uids = $this->entityTypeManager->getStorage('user')->getQuery()
        ->accessCheck(FALSE)
        ->condition('status', 1)
        ->execute();

      $users = $this->entityTypeManager->getStorage('user')->loadMultiple($all_uids);
      foreach ($users as $user) {
        if (!$user->get('field_firebase_tokens')->isEmpty()) {
          $uids[] = $user->id();
        }
      }
    }

    if ($type === 'specific_users') {
      $uids = $target_ids;
    }

    if ($type === 'roles') {
      $query = $this->entityTypeManager->getStorage('user')->getQuery()
        ->accessCheck(FALSE)
        ->condition('status', 1);
      $or = $query->orConditionGroup();
      foreach ($target_ids as $role) {
        $or->condition('roles', $role);
      }
      $uids = $query->condition($or)->execute();
    }

    return !empty($uids)
      ? $this->entityTypeManager->getStorage('user')->loadMultiple($uids)
      : [];
  }

}
