<?php

namespace Drupal\firebase_ui\Service;

use Google\Auth\Credentials\ServiceAccountCredentials;
use GuzzleHttp\Client;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Psr\Log\LoggerInterface;

/**
 * Service to send push notifications via Firebase Cloud Messaging (FCM).
 */
class FirebaseNotificationSender {

  /**
   * Logger for firebase_ui.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Firebase UI config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * Constructs the FirebaseNotificationSender service.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    FileSystemInterface $file_system,
  ) {
    $this->logger = $logger_factory->get('firebase_ui');
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory->get('firebase_ui.settings');
    $this->fileSystem = $file_system;
  }

  /**
   * Sends a notification using Firebase Cloud Messaging.
   *
   * @param string $token
   *   The device FCM token.
   * @param string $title
   *   The notification title.
   * @param string $message
   *   The notification message body.
   * @param array $data
   *   Optional data payload.
   *
   * @return array|bool
   *   FCM response or FALSE on failure.
   */
  public function sendNotification(string $token, string $title, string $message, array $data = []): array|bool {
    try {
      $fid = $this->config->get('service_account_file');

      if (!$fid) {
        throw new \Exception('Service account file ID is not configured.');
      }

      $file = $this->entityTypeManager->getStorage('file')->load($fid);
      if (!$file) {
        throw new \Exception('Firebase service account file not found.');
      }

      $serviceAccountPath = $this->fileSystem->realpath($file->getFileUri());

      $credentials = new ServiceAccountCredentials(
        'https://www.googleapis.com/auth/firebase.messaging',
        $serviceAccountPath,
      );
      $accessToken = $credentials->fetchAuthToken()['access_token'];

      $client = new Client([
        'base_uri' => 'https://fcm.googleapis.com/v1/',
        'headers' => [
          'Authorization' => 'Bearer ' . $accessToken,
          'Content-Type' => 'application/json',
        ],
      ]);

      $projectId = $this->config->get('projectId');

      $payload = [
        'message' => [
          'token' => $token,
          'notification' => [
            'title' => $title,
            'body' => $message,
          ],
          'data' => $data,
        ],
      ];

      $response = $client->post("projects/{$projectId}/messages:send", [
        'body' => json_encode($payload),
      ]);

      $result = json_decode($response->getBody(), TRUE);

      // If token is invalid, remove it.
      if (isset($result['error']['status']) && in_array($result['error']['status'], ['INVALID_ARGUMENT', 'UNREGISTERED'], TRUE)) {
        $this->removeTokenFromUser($token);
        return FALSE;
      }

      $this->logger->info('FCM success: @response', ['@response' => print_r($result, TRUE)]);
      return $result;
    }
    catch (\Exception $e) {
      $this->logger->error('FCM error: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }

  /**
   * Removes a token from all users that have it stored.
   *
   * @param string $token
   *   The device token to remove.
   */
  private function removeTokenFromUser(string $token): void {
    $uids = $this->entityTypeManager
      ->getStorage('user')
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('field_firebase_tokens', $token)
      ->execute();

    if (!empty($uids)) {
      $users = $this->entityTypeManager->getStorage('user')->loadMultiple($uids);
      foreach ($users as $user) {
        $tokens = $user->get('field_firebase_tokens')->getValue();
        $filtered = array_filter($tokens, fn($item) => $item['value'] !== $token);
        $user->set('field_firebase_tokens', $filtered);
        $user->save();
      }
    }
  }

}
